-------------------------------------------------------------------------------
-- An example circuit type DM74185 --> Named as Bin_BCD_6bit
-- VHDL plan B (behavioural, high-level, algorithmic) design 

-- File adapted from HEX_7SEG_decoder tutorial Plan B in P2
-- Translation from sketch diagram in the project specifications (Fig. 4):
-- https://digsys.upc.edu/csd/P02/Bin_BCD_6bitB/bin_BCD_6bit.html  
---------------------------------------------------
-- Exercise P2  - CSD - Design of standard circuits
-- http://digsys.upc.edu
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;

entity Bin_BCD_6bit is

	port (
		E_L		: 	in STD_LOGIC;
		B		: 	in STD_LOGIC_VECTOR(5 DOWNTO 0); 
		T,U		: 	out STD_LOGIC_VECTOR(3 DOWNTO 0)
	);

end Bin_BCD_6bit;
	
architecture Truth_table_description of Bin_BCD_6bit is

	SIGNAL	Y_out		: STD_LOGIC_VECTOR (7 DOWNTO 0);

BEGIN
	PROCESS (E_L, B)  
		BEGIN
			IF E_L = '0' THEN
				CASE B IS    		
					WHEN "000000" =>       		-- 0
						Y_out <= "00000000";
					WHEN "000001" =>       		-- 1
						Y_out <= "00000001";
					WHEN "000010" =>       		-- 2
						Y_out <= "00000010";
					WHEN "000011" =>       		-- 3
						Y_out <= "00000011";
					WHEN "000100" =>       		-- 4
						Y_out <= "00000100";
					WHEN "000101" =>       		-- 5
						Y_out <= "00000101";
					WHEN "000110" =>       		-- 6
						Y_out <= "00000110";
					WHEN "000111" =>       		-- 7
						Y_out <= "00000111";
					WHEN "001000" =>       		-- 8
						Y_out <= "00001000";
					WHEN "001001" =>       		-- 9
						Y_out <= "00001001";

					WHEN "001010" =>       	-- 10
						Y_out <= "00010000";
					WHEN "001011" =>       		-- 11
						Y_out <= "00010001";
					WHEN "001100" =>       		-- 12
						Y_out <= "00010010";
					WHEN "001101" =>       	-- 13
						Y_out <= "00010011";
					WHEN "001110" =>       		-- 14
						Y_out <= "00010100";
					WHEN "001111" =>       		-- 15
						Y_out <= "00010101";
					WHEN "010000" =>       		-- 16
						Y_out <= "00010110";
					WHEN "010001" =>       		-- 17
						Y_out <= "00010111";
					WHEN "010010" =>       		-- 18
						Y_out <= "00011000";
					WHEN "010011" =>       		-- 19
						Y_out <= "00011001";

					WHEN "010100" =>       		-- 20
						Y_out <= "00100000";
					WHEN "010101" =>       		-- 21
						Y_out <= "00100001";
					WHEN "010110" =>       		-- 22
						Y_out <= "00100010";
					WHEN "010111" =>       		-- 23
						Y_out <= "00100011";
					WHEN "011000" =>       		-- 24
						Y_out <= "00100100";
					WHEN "011001" =>       		-- 25
						Y_out <= "00100101";
					WHEN "011010" =>       		-- 26
						Y_out <= "00100110";
					WHEN "011011" =>       		-- 27
						Y_out <= "00100111";
					WHEN "011100" =>       		-- 28
						Y_out <= "00101000";
					WHEN "011101" =>       		-- 29
						Y_out <= "00101001";
						
					WHEN "011110" =>       		-- 30
						Y_out <= "00110000";
					WHEN "011111" =>       		-- 31
						Y_out <= "00110001";

					WHEN "100000" =>       		-- 32
						Y_out <= "00110010";
					WHEN "100001" =>       		-- 33
						Y_out <= "00110011";
					WHEN "100010" =>       		-- 34
						Y_out <= "00110100";
					WHEN "100011" =>       		-- 35
						Y_out <= "00110101";
					WHEN "100100" =>       		-- 36
						Y_out <= "00110110";
					WHEN "100101" =>       		-- 37
						Y_out <= "00110111";
					WHEN "100110" =>       		-- 38
						Y_out <= "00111000";
					WHEN "100111" =>       		-- 39
						Y_out <= "00111001";
					WHEN "101000" =>       		-- 40
						Y_out <= "01000000";
					WHEN "101001" =>       		-- 41
						Y_out <= "01000001";
					WHEN "101010" =>       	-- 42
						Y_out <= "01000010";
					WHEN "101011" =>       		-- 43
						Y_out <= "01000011";
					WHEN "101100" =>       		-- 44
						Y_out <= "01000100";
					WHEN "101101" =>       	-- 45
						Y_out <= "01000101";
					WHEN "101110" =>       		-- 46
						Y_out <= "01000110";
					WHEN "101111" =>       		-- 47
						Y_out <= "01000111";
					WHEN "110000" =>       		-- 48
						Y_out <= "01001000";
					WHEN "110001" =>       		-- 49
						Y_out <= "01001001";
						
					WHEN "110010" =>       		-- 50
						Y_out <= "01010000";
					WHEN "110011" =>       		-- 51
						Y_out <= "01010001";
					WHEN "110100" =>       		-- 52
						Y_out <= "01010010";
					WHEN "110101" =>       		-- 53
						Y_out <= "01010011";
					WHEN "110110" =>       		-- 54
						Y_out <= "01010100";
					WHEN "110111" =>       		-- 55
						Y_out <= "01010101";
					WHEN "111000" =>       		-- 56
						Y_out <= "01010110";
					WHEN "111001" =>       		-- 57
						Y_out <= "01010111";
					WHEN "111010" =>       		-- 58
						Y_out <= "01011000";
					WHEN "111011" =>       		-- 59
						Y_out <= "01011001";

					WHEN "111100" =>       		-- 60
						Y_out <= "01100000";
					WHEN "111101" =>       		-- 61
						Y_out <= "01100001";					
					WHEN "111110" =>       		-- 62
						Y_out <= "01100010";

					WHEN Others =>       		-- 63
						Y_out <= "01100011";
				END CASE;
			
			ELSE 
				Y_out <= "11011101";  -- this is "d d" disabled
			END IF;
	
	END PROCESS;
			
-- Extra logic (simple buffers in this case)

T <= Y_out(7 downto 4);
U <= Y_out(3 downto 0);
				 		 							
end Truth_table_description;
								
