-------------------------------------------------------------------------------- 
-- UPC - EETAC - EEL - CSD - DIGSYS       
--------------------------------------------------------------------------------
-- P8. Tutorial on the structure of a CLK generator:
-- Example application to be used in the LAB5 demonstration board.
-- https://digsys.upc.edu/csd/P05/lab05/lab5.html#prototyping

-- The MAX10 Lite board crystal oscillator chip is OSC_CLK_in = 50 MHz
-- CLK_A_SQ, this is a square waveform of 10 Hz 
-- CLK_B_SQ, this is a square waveform of 1 Hz 

-- Plan: Frequency dividers and T_FF to square the pulsed terminal count signals    
-- Count Enable (CE) and asynchronous active-high clear direct (CD)
-- 2 cascaded frequency dividers in series 
-- 2 T_FF to square the pulsed waveforms 
-- The reference schematic is presented in lecture: 
-- https://digsys.upc.edu/csd/P08/L08_2/L8_2.html 
--------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
	
ENTITY CLK_Generator IS  
      PORT(	CD,CE			: 	IN  std_logic;
			OSC_CLK_in		: 	IN  std_logic;
			CLK_A_SQ		:	OUT std_logic;
			CLK_B_SQ		:	OUT std_logic
          );
END CLK_Generator ;

ARCHITECTURE schematic OF CLK_Generator IS  
-- Components
	COMPONENT Freq_div_2500000 IS                 
		PORT(	CD,CLK,CE	: 	IN  std_logic;
				TC2500000		:	OUT std_logic
		);
	END COMPONENT;	

	COMPONENT Freq_div_10 IS                 
		PORT(	CD,CLK,CE	: 	IN  std_logic;
				TC10		:	OUT std_logic
		);
	END COMPONENT;	

	COMPONENT T_FF IS
		Port (	
			CLK 	: IN	STD_LOGIC;
			CD 		: IN	STD_LOGIC;
			T		: IN	STD_LOGIC;
			Q 		: OUT	STD_LOGIC
		);
	END COMPONENT;					

-- Signals for connecting components together (just the internal wires)
	SIGNAL CLK			:	std_logic;
	SIGNAL Pulsed_2A	:	std_logic;
	SIGNAL Pulsed_2B	:	std_logic;
		
	BEGIN  
-- Instantiation of components

	Chip1	: 	Freq_Div_2500000
	PORT MAP (	   	
-- from component name 	=> to signal or port name
			CLK		=> CLK,
			CD 		=> CD,
			CE 		=> CE,
			TC2500000	=> Pulsed_2A					  
		);

	Chip2	:  Freq_Div_10
		PORT MAP (
-- from component name 	=> to signal or port name
			CLK		=> CLK,
			CD 		=> CD,
			CE 		=> Pulsed_2A,
			TC10 	=> Pulsed_2B					  
		);

	Chip3	: T_FF
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_2A,
			Q 		=> CLK_A_SQ				  
		);		

	Chip4	: T_FF
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_2B,
			Q 		=> CLK_B_SQ					  
		);		
			
-- Connections and logic between components

	-- The internal CLK form the external quartz crystal 
	CLK <= OSC_CLK_in;
	
END schematic ;
